From: Mike Galbraith <efault@gmx.de>
To: Brian Rogers <brian@xyzw.org>
Cc: linux-kernel@vger.kernel.org, Ingo Molnar <mingo@elte.hu>
Subject: [patch take 2] Re: [BUG] How to get real-time priority using idle priority
Date: Mon, 12 Jan 2009 21:46:52 +0100 [thread overview]
Message-ID: <1231793212.5940.2.camel@marge.simson.net> (raw)
In-Reply-To: <1231765433.5789.35.camel@marge.simson.net>
On Mon, 2009-01-12 at 14:03 +0100, Mike Galbraith wrote:
> Would you mind trying the below?
Or better, this more complete version.
Impact: fix latency issues when SCHED_IDLE tasks are queued.
Exclude SCHED_IDLE tasks from wakeup preemption, ensure that same will
always be wakeup preempted, and exclude them from being buddies so they
will only be selected via their vruntime. Per a tip from Peter Zijlstra,
also scale migration vruntime adjustment to compensate for differences
in min_vruntime advance rates.
Reported-by: Brian Rogers <brian@xyzw.org>
Signed-off-by: Mike Galbraith <efault@gmx.de>
kernel/sched.c | 19 +++++++++++++++++--
kernel/sched_fair.c | 22 ++++++++++++++++------
2 files changed, 33 insertions(+), 8 deletions(-)
diff --git a/kernel/sched.c b/kernel/sched.c
index deb5ac8..bdc0b38 100644
--- a/kernel/sched.c
+++ b/kernel/sched.c
@@ -1888,8 +1888,23 @@ void set_task_cpu(struct task_struct *p, unsigned int new_cpu)
schedstat_inc(p, se.nr_forced2_migrations);
}
#endif
- p->se.vruntime -= old_cfsrq->min_vruntime -
- new_cfsrq->min_vruntime;
+ if (old_cpu != new_cpu) {
+ s64 delta = p->se.vruntime - old_cfsrq->min_vruntime;
+
+ /*
+ * min_vruntimes may be advancing at wildly different
+ * rates, so we must scale the delta accordingly.
+ */
+ if (new_cfsrq->load.weight != old_cfsrq->load.weight) {
+ int negative = delta < 0;
+
+ delta = negative ? -delta : delta;
+ delta = calc_delta_mine(delta,
+ new_cfsrq->load.weight, &old_cfsrq->load);
+ delta = negative ? -delta : delta;
+ }
+ p->se.vruntime = new_cfsrq->min_vruntime + delta;
+ }
__set_task_cpu(p, new_cpu);
}
diff --git a/kernel/sched_fair.c b/kernel/sched_fair.c
index 8e1352c..500ed14 100644
--- a/kernel/sched_fair.c
+++ b/kernel/sched_fair.c
@@ -1340,14 +1340,18 @@ wakeup_preempt_entity(struct sched_entity *curr, struct sched_entity *se)
static void set_last_buddy(struct sched_entity *se)
{
- for_each_sched_entity(se)
- cfs_rq_of(se)->last = se;
+ for_each_sched_entity(se) {
+ if (likely(task_of(se)->policy != SCHED_IDLE))
+ cfs_rq_of(se)->last = se;
+ }
}
static void set_next_buddy(struct sched_entity *se)
{
- for_each_sched_entity(se)
- cfs_rq_of(se)->next = se;
+ for_each_sched_entity(se) {
+ if (likely(task_of(se)->policy != SCHED_IDLE))
+ cfs_rq_of(se)->next = se;
+ }
}
/*
@@ -1393,12 +1397,18 @@ static void check_preempt_wakeup(struct rq *rq, struct task_struct *p, int sync)
return;
/*
* Batch tasks do not preempt (their preemption is driven by
* the tick):
*/
- if (unlikely(p->policy == SCHED_BATCH))
+ if (unlikely(p->policy != SCHED_NORMAL))
return;
+ /* Idle tasks are by definition preempted by everybody. */
+ if (unlikely(curr->policy == SCHED_IDLE)) {
+ resched_task(curr);
+ return;
+ }
+
if (!sched_feat(WAKEUP_PREEMPT))
return;
next prev parent reply other threads:[~2009-01-12 20:47 UTC|newest]
Thread overview: 24+ messages / expand[flat|nested] mbox.gz Atom feed top
2009-01-11 10:58 Brian Rogers
2009-01-12 5:09 ` Mike Galbraith
2009-01-12 13:03 ` Mike Galbraith
2009-01-12 13:14 ` Ingo Molnar
2009-01-12 15:23 ` Mike Galbraith
2009-01-12 15:24 ` Ingo Molnar
2009-01-13 1:05 ` Brian Rogers
2009-01-13 2:58 ` Mike Galbraith
2009-01-14 5:13 ` Mike Galbraith
2009-01-14 5:31 ` Ingo Molnar
2009-01-14 6:02 ` Mike Galbraith
2009-01-14 7:35 ` Ingo Molnar
2009-01-15 9:28 ` Mike Galbraith
2009-01-15 10:14 ` Peter Zijlstra
2009-01-15 10:30 ` Mike Galbraith
2009-01-15 11:37 ` Mike Galbraith
2009-01-15 11:41 ` Peter Zijlstra
2009-01-15 12:54 ` Ingo Molnar
2009-01-15 13:05 ` Peter Zijlstra
2009-01-15 13:15 ` Ingo Molnar
2009-01-15 13:16 ` Peter Zijlstra
2009-01-15 12:07 ` Brian Rogers
2009-01-12 20:46 ` Mike Galbraith [this message]
2009-01-12 20:50 ` [patch take 2] " Mike Galbraith
Reply instructions:
You may reply publicly to this message via plain-text email
using any one of the following methods:
* Save the following mbox file, import it into your mail client,
and reply-to-all from there: mbox
Avoid top-posting and favor interleaved quoting:
https://en.wikipedia.org/wiki/Posting_style#Interleaved_style
* Reply using the --to, --cc, and --in-reply-to
switches of git-send-email(1):
git send-email \
--in-reply-to=1231793212.5940.2.camel@marge.simson.net \
--to=efault@gmx.de \
--cc=brian@xyzw.org \
--cc=linux-kernel@vger.kernel.org \
--cc=mingo@elte.hu \
/path/to/YOUR_REPLY
https://kernel.org/pub/software/scm/git/docs/git-send-email.html
* If your mail client supports setting the In-Reply-To header
@@ -1393,12 +1397,18 @@ static void check_preempt_wakeup(struct rq *rq, struct task_struct *p, int sync)
return;
/*
- * Batch tasks do not preempt (their preemption is driven by
+ * Batch and idle tasks do not preempt (their preemption is driven by
* the tick):
*/
via mailto: links, try the mailto: link
Be sure your reply has a Subject: header at the top and a blank line
before the message body.
This is a public inbox, see mirroring instructions
for how to clone and mirror all data and code used for this inbox
Powered by JetHome